﻿
namespace Common.IdentityServerRepository.Infrastructure
{
    using Microsoft.AspNetCore.Identity;
    using Microsoft.Extensions.Logging;
    using System;
    using System.Linq;
    using System.Threading.Tasks;    
    using System.Collections.Generic;
    using Interfaces;

    public class ClientManager<TClient, TKey> : IDisposable
        where TClient : class, IClient<TKey>
        where TKey : IEquatable<TKey>
    {
        private bool _disposed = false;
        protected IClientStore<TClient, TKey> Store { get; }        
        protected IServiceProvider Services { get; }
        public virtual IQueryable<TClient> Clients
        {
            get
            {
                var queryableStore = Store as IQueryableClientStore<TClient, TKey>;
                if (queryableStore == null)
                    throw new NotSupportedException("IQueryable<TClient>");                

                return queryableStore.Clients;
            }
        }        
        
        public ClientManager(IClientStore<TClient, TKey> store, IServiceProvider services)
        {
            if (store == null)
                throw new ArgumentNullException("store");

            Store = store;                        
            Services = services;
        }
                
        public virtual async Task<IdentityResult> AddAsync(TClient client)
        {
            ThrowIfDisposed();
            if (client == null)
                throw new ArgumentNullException("client");            

            Store.Add(client);
            await Store.SaveAllChangesAsync();
            return IdentityResult.Success;
        }                
        public virtual async Task<IdentityResult> RemoveAsync(TClient client)
        {
            ThrowIfDisposed();
            if (client == null)
                throw new ArgumentNullException("client");

            Store.Remove(client);
            await Store.SaveAllChangesAsync();
            return IdentityResult.Success;
        }
        public virtual async Task<IdentityResult> UpdateAsync(TClient client)
        {
            ThrowIfDisposed();
            if (client == null)
                throw new ArgumentNullException("client");

            Store.Modify(client);
            await Store.SaveAllChangesAsync();
            return IdentityResult.Success;
        }
        public virtual async Task<TClient> FindByIdAsync(TKey clientId)
        {
            ThrowIfDisposed();
            return await Store.FindByIdAsync(clientId);

        }
        public virtual async Task<List<TClient>> FindByUserAsync(TKey userId)
        {
            ThrowIfDisposed();
            return await Store.FindByUserAsync(userId);
        }               
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }
        protected virtual void Dispose(bool disposing)
        {
            if (disposing && !_disposed)
            {
                Store.Dispose();
            }
            _disposed = true;
        }
        private void ThrowIfDisposed()
        {
            if (_disposed)
                throw new ObjectDisposedException(GetType().Name);
        }
    }
}
