﻿
namespace Common.IdentityServerRepository.Infrastructure
{
    using Microsoft.AspNetCore.Identity;
    using Microsoft.Extensions.Logging;
    using System;
    using System.Linq;
    using System.Threading.Tasks;
    using Interfaces;    

    public class SecretManager<TSecret, TKey> : IDisposable
        where TSecret : class, ISecret<TKey>
        where TKey : IEquatable<TKey>
    {
        private bool _disposed = false;

        protected ISecretStore<TSecret, TKey> Store { get; }
        protected ILogger<SecretManager<TSecret, TKey>> Logger { get; }
        public virtual IQueryable<TSecret> Secrets
        {
            get
            {
                var queryableStore = Store as IQueryableSecretStore<TSecret, TKey>;
                if (queryableStore == null)
                    throw new NotSupportedException("IQueryable<TSecret>");

                return queryableStore.Secrets;
            }
        }
        public IEmailService EmailService { get; set; }

        public SecretManager(ISecretStore<TSecret, TKey> store, ILogger<SecretManager<TSecret, TKey>> logger)
        {
            if (store == null)
                throw new ArgumentNullException("store");

            Store = store;
            Logger = logger;
        }

        public virtual async Task<IdentityResult> CreateAsync(TSecret secret)
        {
            ThrowIfDisposed();
            if (secret == null)
                throw new ArgumentNullException("client");

            Store.Add(secret);
            await Store.SaveAllChangesAsync();
            return IdentityResult.Success;           
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }
        protected virtual void Dispose(bool disposing)
        {
            if (disposing && !_disposed)
            {
                Store.Dispose();
            }
            _disposed = true;
        }
        private void ThrowIfDisposed()
        {
            if (_disposed)
                throw new ObjectDisposedException(GetType().Name);
        }
    }
}
