﻿
namespace Common.IdentityServerRepository.Stores
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Threading.Tasks;
    using Infrastructure;
    using Interfaces;
    using Microsoft.EntityFrameworkCore;

    public class ClientSecretStore<TClientSecret, TKey> : IQueryableClientSecretStore<TClientSecret, TKey>, IDisposable
        where TClientSecret : IdentityClientSecret<TKey>, new()
        where TKey : IEquatable<TKey>, IComparable<TKey>
    {
        private bool _disposed = false;
        public DbContext Context { get; private set; }
        public bool DisposeContext { get; set; }
        public IQueryable<TClientSecret> ClientsSecrets { get { return Context.Set<TClientSecret>(); } }

        public ClientSecretStore(DbContext context)
        {
            if (context == null)
                throw new ArgumentNullException("context");

            Context = context;
        }
        public virtual void Add(TClientSecret clientSecret)
        {
            ThrowIfDisposed();
            if (clientSecret == null)
                throw new ArgumentNullException("clientSecret");

            var dbEntityEntry = Context.Entry<TClientSecret>(clientSecret);
            Context.Set<TClientSecret>().Add(clientSecret);
            //return Context.SaveChangesAsync();
        }
        public virtual void Remove(TClientSecret clientSecret)
        {
            ThrowIfDisposed();
            if (clientSecret == null)
                throw new ArgumentNullException("client");

            var dbEntityEntry = Context.Entry<TClientSecret>(clientSecret);
            dbEntityEntry.State = EntityState.Deleted;
            //return Context.SaveChangesAsync();
        }
        public Task<List<TClientSecret>> FindByClientAsync(TKey clientId)
        {
            ThrowIfDisposed();
            return Context.Set<TClientSecret>().Where(w => w.ClientId.Equals(clientId)).ToListAsync();
        }
        public Task<List<TClientSecret>> FindBySecretAsync(TKey secretId)
        {
            ThrowIfDisposed();
            return Context.Set<TClientSecret>().Where(w => w.SecretId.Equals(secretId)).ToListAsync();
        }
        public Task<int> SaveAllChangesAsync()
        {
            return Context.SaveChangesAsync();
        }
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }       
        protected virtual void Dispose(bool disposing)
        {
            if (DisposeContext && disposing && Context != null)
            {
                Context.Dispose();
            }

            _disposed = true;
            Context = null;
        }
        private void ThrowIfDisposed()
        {
            if (_disposed)
            {
                throw new ObjectDisposedException(GetType().Name);
            }
        }        
    }
}
