﻿
namespace Common.IdentityServerRepository.Stores
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Threading.Tasks;
    using Infrastructure;
    using Interfaces;
    using Microsoft.EntityFrameworkCore;
    using Microsoft.EntityFrameworkCore.ChangeTracking;

    public class ClientStore<TClient, TKey> : IQueryableClientStore<TClient, TKey>,  IDisposable
        where TClient : IdentityClient<TKey>, new()
        where TKey : IEquatable<TKey>
    {
        private bool _disposed = false;
        public DbContext Context { get; private set; }
        public bool DisposeContext { get; set; }
        public IQueryable<TClient> Clients { get { return Context.Set<TClient>(); } }

        public ClientStore(DbContext context)
        {
            if (context == null)
                throw new ArgumentNullException("context");

            Context = context;
        }

        public virtual void Add(TClient client)
        {
            ThrowIfDisposed();
            if (client == null)
                throw new ArgumentNullException("client");

            var dbEntityEntry = Context.Entry<TClient>(client);
            Context.Set<TClient>().Add(client);
            //return Context.Set<TClient>().Add(client);            
        }
        public virtual void Remove(TClient client)
        {
            ThrowIfDisposed();
            if (client == null)
                throw new ArgumentNullException("client");

            var dbEntityEntry = Context.Entry<TClient>(client);
            dbEntityEntry.State = EntityState.Deleted;            
            //return SaveAllChangesAsync();
        }        
        public virtual void Modify(TClient client)
        {
            ThrowIfDisposed();
            if (client == null)
                throw new ArgumentNullException("client");

            var dbEntityEntry = Context.Entry<TClient>(client);
            dbEntityEntry.State = EntityState.Modified;            
            //return SaveAllChangesAsync();
        }
        public virtual Task<TClient> FindByIdAsync(TKey id)
        {
            ThrowIfDisposed();
            return Context.Set<TClient>().FirstOrDefaultAsync(w => w.Id.Equals(id));
        }
        public virtual Task<List<TClient>> FindByUserAsync(TKey userId)
        {
            ThrowIfDisposed();
            return Context.Set<TClient>().Where(w => w.UserId.Equals(userId)).ToListAsync();
        }
        public Task<int> SaveAllChangesAsync()
        {
            return Context.SaveChangesAsync();
        }
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }
        protected virtual void Dispose(bool disposing)
        {
            if (DisposeContext && disposing && Context != null)
            {
                Context.Dispose();
            }

            _disposed = true;
            Context = null;
        }
        private void ThrowIfDisposed()
        {
            if (_disposed)
            {
                throw new ObjectDisposedException(GetType().Name);
            }
        }               
    }
}
