﻿
namespace Common.IdentityServerRepository.Stores
{
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Threading.Tasks;
    using Infrastructure;
    using Interfaces;
    using Microsoft.EntityFrameworkCore;

    public class SecretStore<TSecret, TKey> : IQueryableSecretStore<TSecret, TKey>, IDisposable
        where TSecret : IdentitySecret<TKey>, new()
        where TKey : IEquatable<TKey>, IComparable<TKey>
    {
        private bool _disposed = false;
        public DbContext Context { get; private set; }
        public bool DisposeContext { get; set; }
        public IQueryable<TSecret> Secrets { get { return Context.Set<TSecret>(); } }

        public SecretStore(DbContext context)
        {
            if (context == null)
                throw new ArgumentNullException("context");

            Context = context;
        }
        public virtual void Add(TSecret secret)
        {
            ThrowIfDisposed();
            if (secret == null)
                throw new ArgumentNullException("secret");

            var dbEntityEntry = Context.Entry<TSecret>(secret);
            Context.Set<TSecret>().Add(secret);
            //return Context.SaveChangesAsync();
        }
        public virtual void Remove(TSecret secret)
        {
            ThrowIfDisposed();
            if (secret == null)
                throw new ArgumentNullException("secret");

            var dbEntityEntry = Context.Entry<TSecret>(secret);
            dbEntityEntry.State = EntityState.Deleted;
            //return Context.SaveChangesAsync();
        }
        public virtual void Modify(TSecret secret)
        {
            ThrowIfDisposed();
            if (secret == null)
                throw new ArgumentNullException("secret");

            var dbEntityEntry = Context.Entry<TSecret>(secret);
            dbEntityEntry.State = EntityState.Modified;
            //return Context.SaveChangesAsync();
        }
        public Task<TSecret> FindByIdAsync(TKey id)
        {
            ThrowIfDisposed();
            return Context.Set<TSecret>().FirstOrDefaultAsync(w => w.Id.Equals(id));
        }
        public Task<int> SaveAllChangesAsync()
        {
            return Context.SaveChangesAsync();
        }
        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }
        protected virtual void Dispose(bool disposing)
        {
            if (DisposeContext && disposing && Context != null)
            {
                Context.Dispose();
            }

            _disposed = true;
            Context = null;
        }
        private void ThrowIfDisposed()
        {
            if (_disposed)
            {
                throw new ObjectDisposedException(GetType().Name);
            }
        }        
    }
}
